@@ -3,21 +3,32 @@ from django.core.urlresolvers import reverse, NoReverseMatch
 from django.utils.safestring import mark_safe
 
 from layout import Layout, LayoutSlice
-from utils import render_field, flatatt
+from utils import render_field, flatatt, TEMPLATE_PACK
 from exceptions import FormHelpersException
 
 
 class DynamicLayoutHandler(object):
+    def _check_layout(self):
+        if self.layout is None:
+            raise FormHelpersException("You need to set a layout in your FormHelper")
+
+    def _check_layout_and_form(self):
+        self._check_layout()
+        if self.form is None:
+            raise FormHelpersException("You need to pass a form instance to your FormHelper")
+
     def all(self):
         """
         Returns all layout objects of first level of depth
         """
+        self._check_layout()
         return LayoutSlice(self.layout, slice(0, len(self.layout.fields), 1))
 
     def filter(self, *LayoutClasses, **kwargs):
         """
         Returns a LayoutSlice pointing to layout objects of type `LayoutClass`
         """
+        self._check_layout()
         max_level = kwargs.pop('max_level', 0)
         filtered_layout_objects = self.layout.get_layout_objects(LayoutClasses, max_level=max_level)
 
@@ -27,7 +38,7 @@ class DynamicLayoutHandler(object):
         """
         Returns a LayoutSlice pointing to fields with widgets of `widget_type`
         """
-        assert(self.layout is not None and self.form is not None)
+        self._check_layout_and_form()
         layout_field_names = self.layout.get_field_names()
 
         # Let's filter all fields with widgets like widget_type
@@ -42,7 +53,7 @@ class DynamicLayoutHandler(object):
         """
         Returns a LayoutSlice pointing to fields with widgets NOT matching `widget_type`
         """
-        assert(self.layout is not None and self.form is not None)
+        self._check_layout_and_form()
         layout_field_names = self.layout.get_field_names()
 
         # Let's exclude all fields with widgets like widget_type
@@ -65,7 +76,7 @@ class DynamicLayoutHandler(object):
             if hasattr(self, key):
                 return getattr(self, key)
 
-            assert(self.layout is not None)
+            self._check_layout()
             layout_field_names = self.layout.get_field_names()
 
             filtered_field = []
@@ -164,8 +175,8 @@ class FormHelper(DynamicLayoutHandler):
     formset_error_title = None
     form_show_errors = True
     render_unmentioned_fields = False
-    help_text_inline = False
-    error_text_inline = True
+    _help_text_inline = False
+    _error_text_inline = True
     html5_required = False
 
     def __init__(self, form=None):
@@ -220,40 +231,63 @@ class FormHelper(DynamicLayoutHandler):
 
     form_style = property(get_form_style, set_form_style)
 
+    def get_help_text_inline(self):
+        return self._help_text_inline
+
+    def set_help_text_inline(self, flag):
+        self._help_text_inline = flag
+        self._error_text_inline = not flag
+
+    help_text_inline = property(get_help_text_inline, set_help_text_inline)
+
+    def get_error_text_inline(self):
+        return self._error_text_inline
+
+    def set_error_text_inline(self, flag):
+        self._error_text_inline = flag
+        self._help_text_inline = not flag
+
+    error_text_inline = property(get_error_text_inline, set_error_text_inline)
+
     def add_input(self, input_object):
         self.inputs.append(input_object)
 
     def add_layout(self, layout):
         self.layout = layout
 
-    def render_layout(self, form, context):
+    def render_layout(self, form, context, template_pack=TEMPLATE_PACK):
         """
         Returns safe html of the rendering of the layout
         """
         form.rendered_fields = set()
 
         # This renders the specifed Layout
-        html = self.layout.render(form, self.form_style, context)
+        html = self.layout.render(form, self.form_style, context,
+                                  template_pack=template_pack)
 
         if self.render_unmentioned_fields:
             fields = set(form.fields.keys())
             left_fields_to_render = fields - form.rendered_fields
             for field in left_fields_to_render:
-                html += render_field(field, form, self.form_style, context)
+                html += render_field(field, form, self.form_style, context, template_pack=template_pack)
 
-        # If the user has meta fields defined, not included in the layout
-        # we suppose they need to be rendered. Othewise we renderd the
+        # If the user has Meta.fields defined, not included in the layout
+        # we suppose they need to be rendered. Otherwise we render the
         # layout fields strictly
-        if getattr(form, 'Meta', None):
-            current_fields = set(getattr(form, 'fields', []))
-            left_fields_to_render = current_fields - form.rendered_fields
+        if hasattr(form, 'Meta'):
+            if hasattr(form.Meta, 'fields'):
+                current_fields = set(getattr(form, 'fields', []))
+                meta_fields = set(getattr(form.Meta, 'fields'))
 
-            for field in left_fields_to_render:
-                html += render_field(field, form, self.form_style, context)
+                fields_to_render = current_fields & meta_fields
+                left_fields_to_render = fields_to_render - form.rendered_fields
+
+                for field in left_fields_to_render:
+                    html += render_field(field, form, self.form_style, context)
 
         return mark_safe(html)
 
-    def get_attributes(self):
+    def get_attributes(self, template_pack=TEMPLATE_PACK):
         """
         Used by crispy_forms_tags to get helper attributes
         """
@@ -275,12 +309,12 @@ class FormHelper(DynamicLayoutHandler):
             items['attrs']['id'] = self.form_id.strip()
         if self.form_class:
             # uni_form TEMPLATE PACK has a uniForm class by default
-            if getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap') == 'uni_form':
+            if template_pack == 'uni_form':
                 items['attrs']['class'] = "uniForm %s" % self.form_class.strip()
             else:
                 items['attrs']['class'] = self.form_class.strip()
         else:
-            if getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap') == 'uni_form':
+            if template_pack == 'uni_form':
                 items['attrs']['class'] = self.attrs.get('class', '') + " uniForm"
 
         items['flat_attrs'] = flatatt(items['attrs'])
