@@ -2,6 +2,7 @@ from itertools import izip
 
 from django import template
 
+
 register = template.Library()
 
 class_converter = {
@@ -18,8 +19,18 @@ def is_checkbox(field):
 def is_password(field):
     return field.field.widget.__class__.__name__.lower() == "passwordinput"
 
+@register.filter
+def classes(field):
+    """
+    Returns CSS classes of a field
+    """
+    return field.widget.attrs.get('class', None)
+
 @register.filter
 def css_class(field):
+    """
+    Returns widgets class name in lowercase
+    """
     return field.field.widget.__class__.__name__.lower()
 
 def pairwise(iterable):
@@ -31,15 +42,24 @@ class CrispyFieldNode(template.Node):
     def __init__(self, field, attrs):
        self.field = field
        self.attrs = attrs
+       self.html5_required = 'html5_required'
 
     def render(self, context):
         # Nodes are not threadsafe so we must store and look up our instance
         # variables in the current rendering context first
         if self not in context.render_context:
-            context.render_context[self] = (template.Variable(self.field), self.attrs,)
+            context.render_context[self] = (
+                template.Variable(self.field),
+                self.attrs,
+                template.Variable(self.html5_required)
+            )
 
-        field, attrs = context.render_context[self]
+        field, attrs, html5_required = context.render_context[self]
         field = field.resolve(context)
+        try:
+            html5_required = html5_required.resolve(context)
+        except template.VariableDoesNotExist:
+            html5_required = False
 
         widgets = getattr(field.field.widget, 'widgets', [field.field.widget,])
 
@@ -50,7 +70,6 @@ class CrispyFieldNode(template.Node):
             class_name = widget.__class__.__name__.lower()
             class_name = class_converter.get(class_name, class_name)
             css_class = widget.attrs.get('class', '')
-
             if css_class:
                 if css_class.find(class_name) == -1:
                     css_class += " %s" % class_name
@@ -59,6 +78,11 @@ class CrispyFieldNode(template.Node):
 
             widget.attrs['class'] = css_class
 
+            # HTML5 required attribute
+            if html5_required and field.field.required and 'required' not in widget.attrs:
+                if field.field.widget.__class__.__name__ is not 'RadioSelect':
+                    widget.attrs['required'] = 'required'
+
             for attribute_name, attribute in attr.items():
                 widget.attrs[template.Variable(attribute_name).resolve(context)] = template.Variable(attribute).resolve(context)
 
