@@ -20,14 +20,15 @@ from crispy_forms.layout import (
     Div, Field, MultiWidgetField
 )
 from crispy_forms.bootstrap import (
-    AppendedPrependedText, AppendedText, PrependedText, InlineCheckboxes
+    AppendedPrependedText, AppendedText, PrependedText, InlineCheckboxes,
+    FieldWithButtons, StrictButton, InlineRadios
 )
 from crispy_forms.utils import render_crispy_form
 from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
 
 from forms import (
     TestForm, TestForm2, TestForm3, ExampleForm, CheckboxesTestForm,
-    FormWithMeta
+    FormWithMeta, TestForm4
 )
 
 
@@ -116,14 +117,10 @@ class TestFormHelpers(TestCase):
 
     def test_inputs(self):
         form_helper = FormHelper()
-        submit  = Submit('my-submit', 'Submit', css_class="button white")
-        reset   = Reset('my-reset', 'Reset')
-        hidden  = Hidden('my-hidden', 'Hidden')
-        button  = Button('my-button', 'Button')
-        form_helper.add_input(submit)
-        form_helper.add_input(reset)
-        form_helper.add_input(hidden)
-        form_helper.add_input(button)
+        form_helper.add_input(Submit('my-submit', 'Submit', css_class="button white"))
+        form_helper.add_input(Reset('my-reset', 'Reset'))
+        form_helper.add_input(Hidden('my-hidden', 'Hidden'))
+        form_helper.add_input(Button('my-button', 'Button'))
 
         template = get_template_from_string(u"""
             {% load crispy_forms_tags %}
@@ -146,7 +143,7 @@ class TestFormHelpers(TestCase):
             self.assertTrue('class="btn"' in html)
             self.assertTrue('btn btn-primary' in html)
             self.assertTrue('btn btn-inverse' in html)
-            self.assertEqual(html.count('/>&zwnj;'), 4)
+            self.assertEqual(len(re.findall(r'<input[^>]+> <', html)), 8)
 
     def test_invalid_form_method(self):
         form_helper = FormHelper()
@@ -240,6 +237,7 @@ class TestFormHelpers(TestCase):
             AppendedText('email', 'whatever'),
             PrependedText('first_name', 'blabla'),
             AppendedPrependedText('last_name', 'foo', 'bar'),
+            MultiField('legend', 'password1', 'password2')
         )
         form.is_valid()
 
@@ -251,6 +249,30 @@ class TestFormHelpers(TestCase):
         html = render_crispy_form(form)
         self.assertEqual(html.count('error'), 0)
 
+    def test_multifield_errors(self):
+        form = TestForm({
+            'email': 'invalidemail',
+            'password1': 'yes',
+            'password2': 'yes',
+        })
+        form.helper = FormHelper()
+        form.helper.layout = Layout(
+            MultiField('legend', 'email')
+        )
+        form.is_valid()
+
+        form.helper.form_show_errors = True
+        html = render_crispy_form(form)
+        self.assertEqual(html.count('error'), 3)
+
+        # Reset layout for avoiding side effects
+        form.helper.layout = Layout(
+            MultiField('legend', 'email')
+        )
+        form.helper.form_show_errors = False
+        html = render_crispy_form(form)
+        self.assertEqual(html.count('error'), 0)
+
     def test_html5_required(self):
         form = TestForm()
         form.helper = FormHelper()
@@ -382,6 +404,33 @@ class TestFormHelpers(TestCase):
         if (settings.CRISPY_TEMPLATE_PACK == 'uni_form'):
             self.assertTrue('uniForm' in html)
 
+    def test_template_pack_override(self):
+        current_pack = settings.CRISPY_TEMPLATE_PACK
+        override_pack = current_pack == 'uni_form' and 'bootstrap' or 'uni_form'
+
+        # Syntax {% crispy form 'template_pack_name' %}
+        template = get_template_from_string(u"""
+            {%% load crispy_forms_tags %%}
+            {%% crispy form "%s" %%}
+        """ % override_pack)
+        c = Context({'form': TestForm()})
+        html = template.render(c)
+
+        # Syntax {% crispy form helper 'template_pack_name' %}
+        template = get_template_from_string(u"""
+            {%% load crispy_forms_tags %%}
+            {%% crispy form form_helper "%s" %%}
+        """ % override_pack)
+        c = Context({'form': TestForm(), 'form_helper': FormHelper()})
+        html2 = template.render(c)
+
+        if (current_pack == 'uni_form'):
+            self.assertTrue('control-group' in html)
+            self.assertTrue('control-group' in html2)
+        else:
+            self.assertTrue('uniForm' in html)
+            self.assertTrue('uniForm' in html2)
+
     def test_invalid_helper(self):
         template = get_template_from_string(u"""
             {% load crispy_forms_tags %}
@@ -550,9 +599,9 @@ class TestFormLayout(TestCase):
 
     def test_hidden_fields(self):
         form = TestForm()
-        # All fields fake hidden
+        # All fields hidden
         for field in form.fields:
-            form[field].field.widget.is_hidden = True
+            form.fields[field].widget = forms.HiddenInput()
 
         form.helper = FormHelper()
         form.helper.layout = Layout(
@@ -560,9 +609,42 @@ class TestFormLayout(TestCase):
             PrependedText('password2', 'bar'),
             AppendedPrependedText('email', 'bar'),
             InlineCheckboxes('first_name'),
+            InlineRadios('last_name'),
         )
         html = render_crispy_form(form)
-        self.assertEqual(html.count("<input"), 4)
+        self.assertEqual(html.count("<input"), 5)
+        self.assertEqual(html.count('type="hidden"'), 5)
+
+    def test_field_with_buttons(self):
+        form = TestForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(
+            FieldWithButtons(
+                Field('password1', css_class="span4"),
+                StrictButton("Go!", css_id="go-button"),
+                StrictButton("No!", css_class="extra"),
+                StrictButton("Test", type="submit", name="whatever", value="something"),
+                css_class="extra",
+                autocomplete="off"
+            )
+        )
+        html = render_crispy_form(form)
+        self.assertEqual(html.count('class="control-group extra"'), 1)
+        self.assertEqual(html.count('autocomplete="off"'), 1)
+        self.assertEqual(html.count('class="input-append"'), 1)
+        self.assertEqual(html.count('class="span4'), 1)
+        self.assertEqual(html.count('id="go-button"'), 1)
+        self.assertEqual(html.count("Go!"), 1)
+        self.assertEqual(html.count("No!"), 1)
+        self.assertEqual(html.count('class="btn"'), 2)
+        self.assertEqual(html.count('class="btn extra"'), 1)
+        self.assertEqual(html.count('type="submit"'), 1)
+        self.assertEqual(html.count('name="whatever"'), 1)
+        self.assertEqual(html.count('value="something"'), 1)
+
+        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+            # Make sure white spaces between buttons are there in bootstrap
+            self.assertEqual(len(re.findall(r'</button> <', html)), 3)
 
     def test_layout_fieldset_row_html_with_unicode_fieldnames(self):
         form_helper = FormHelper()
@@ -874,11 +956,20 @@ class TestFormLayout(TestCase):
         test_form = TestForm3()
         self.assertEqual(test_form.helper.layout.fields, ['email'])
 
+    def test_modelform_layout_without_meta(self):
+        test_form = TestForm4()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout('email')
+        html = render_crispy_form(test_form)
+
+        self.assertTrue('email' in html)
+        self.assertFalse('password' in html)
+
     def test_multiplecheckboxes(self):
         test_form = CheckboxesTestForm()
         html = render_crispy_form(test_form)
 
-        self.assertEqual(html.count('checked="checked"'), 5)
+        self.assertEqual(html.count('checked="checked"'), 6)
 
         test_form.helper = FormHelper(test_form)
         test_form.helper[1].wrap(InlineCheckboxes, inline=True)
@@ -933,10 +1024,18 @@ class TestLayoutObjects(TestCase):
         html = template.render(c)
 
         # Check form parameters
-        self.assertEqual(html.count('<span class="add-on ">@</span>'), 1)
-        self.assertEqual(html.count('<span class="add-on ">gmail.com</span>'), 1)
-        self.assertEqual(html.count('<span class="add-on ">#</span>'), 1)
-        self.assertEqual(html.count('<span class="add-on ">$</span>'), 1)
+        self.assertEqual(html.count('<span class="add-on">@</span>'), 1)
+        self.assertEqual(html.count('<span class="add-on">gmail.com</span>'), 1)
+        self.assertEqual(html.count('<span class="add-on">#</span>'), 1)
+        self.assertEqual(html.count('<span class="add-on">$</span>'), 1)
+
+    def test_inline_radios(self):
+        test_form = CheckboxesTestForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(InlineRadios('inline_radios'))
+        html = render_crispy_form(test_form)
+
+        self.assertEqual(html.count('radio inline"'), 2)
 
 
 class TestDynamicLayouts(TestCase):
@@ -1282,6 +1381,17 @@ class TestDynamicLayouts(TestCase):
         self.assertTrue(isinstance(form.helper.layout[0][1][0][0], basestring))
         self.assertTrue(isinstance(form.helper.layout[0][4][0], basestring))
 
+    def test_all_without_layout(self):
+        form = TestForm()
+        form.helper = FormHelper()
+        self.assertRaises(FormHelpersException, lambda: form.helper.all().wrap(Div))
+
+    def test_filter_by_widget_without_form(self):
+        form = TestForm()
+        form.helper = FormHelper()
+        form.helper.layout = self.advanced_layout
+        self.assertRaises(FormHelpersException, lambda: form.helper.filter_by_widget(forms.PasswordInput))
+
     def test_getitem_by_field_name(self):
         form = TestForm()
         form.helper = FormHelper(form)
